#!/usr/bin/env python

from I3Tray import *
from I3Units import *
from math import *
from os.path import expandvars

import os

eve_num = 100
outgcdfile  = "/icrr/work/icecube/data/MCData/40strings/V02-00-14/juliet/test/i3_files/GCD_40str.i3.gz"
physfile  = "nu_e40strTest.i3.gz"
outrootfile = "nu_e40strTest.root"
number = 23
if len(sys.argv) > 1:
	physfile = sys.argv[1]
if len(sys.argv) > 2:
	outrootfile = sys.argv[2]
if len(sys.argv) > 3:
	number = int(sys.argv[3])

load("libc2j-icetray")
load("libicetray")
load("libdataclasses")
load('libdataio')
load("libphys-services")
load("libsim-services")
load("libjuliet-interface")
load("libromeo")
load("libPSInterface")
load("libDOMcalibrator")
load("libDOMsimulator")
load("libpmt-simulator")
load("libhit-constructor")
load("libPSInterfaceFactory")
load("libbrutus")
load("libportia")
load("libophelia")
load("libweighting-module")
load("libcmc")
load("libDomTools")

tray = I3Tray()

gcd_file = expandvars("$I3_BUILD/sample/GCD/GCD.i3.gz")
photonics = expandvars("$I3_BUILD/")
stringsToUse  = "21,29,30,38,39,40,49,50,59,58,67,66,74,73,65,72,78,48,57,47,46,56,63,64,55,71,70,76,77,75,69,60,68,61,62,52,44,53,54,45"
stationsToUse = "21,29,30,38,39,40,49,50,59,58,67,66,74,73,65,72,78,48,57,47,46,56,63,64,55,71,70,76,77,75,69,60,68,61,62,52,44,53,54,45"

tray.AddService("I3JavaVMFactory", "java_vm")(
        ("Options",[expandvars("-Djava.class.path=$I3_SRC/juliet/java_lib/classes"), "-Xms256m", "-Xmx712m"])
	)

tray.AddService("I3MCTimeGeneratorServiceFactory","time-gen")(
	("Year",2007),
        ("DAQTime",173356670000000001)
	)

tray.AddService("I3ReaderServiceFactory","reader")(
	("Filename",gcd_file),
	("OmitEvent",True)
	)

tray.AddService("I3GeometrySelectorServiceFactory","geo-selector")(
	("StringsToUse",stringsToUse),
	("StationsToUse",stationsToUse),
	#("ShiftToCenter",True),
	("GeoSelectorName","IC22-IT26-Geo")
	)

tray.AddService("I3PSInterfaceFactory","PSInterface")(
	("Interface","Photonics"),
	("TablesDirectory",photonics),
        ("DriverFilePath",photonics),
	("PhotonicsLevel1DriverFile","level1_table.list"),
	("PhotonicsLevel2DriverFile","level2_table.list"),
	("PhotonicsTableSelection",1) # cascade only
	)

stream_num=number%10
seed=(number-stream_num)/10
tray.AddService("I3SPRNGRandomServiceFactory","random")(
        ("Seed",seed),
        ("NStreams",100),
        ("StreamNum",stream_num))

tray.AddModule("I3Muxer","muxer")(
        ("GeometryService","IC22-IT26-Geo")
	)

tray.AddModule("I3JulietPrimaryParticleSource","fillprimary")(
	("PrimaryFlavor",0),  #[e, mu, tau]=[0, 1, 2]
	("PrimaryDoublet",0), #[neutrino, charged]=[0, 1]
	("EnergyMin", 1.E5*I3Units.GeV),  #[GeV]
	("EnergyMax", 1.E11*I3Units.GeV),
	("DifferentialPowerLawIndex", 1.),
	("InjectPositionX",310.*I3Units.m),
	("InjectPositionY",189.*I3Units.m),
	("InjectPositionZ",0.*I3Units.m),
	("InjectRadius",880.*I3Units.m),
	("NadirMin",0.*I3Units.deg),
	("NadirMax",180.*I3Units.deg),
	("AzimuthMin",0.*I3Units.deg),
	("AzimuthMax",360.*I3Units.deg),
	("NeutrinoInteractionWeight", 1)
	)

tray.AddModule("I3Juliet","propagate")(
	("MaterialID",0),  #[ice, rock]=[0, 1]
	("Flavor",0),      #[e, mu, tau]=[0, 1, 2]
	#This should be same as the PrimaryFlavor
	("DoChargedCurrent",1),
	("DoNeutralCurrent",1),
	("DoMuBremss",1),
	("DoTauBremss",1),
	("DoMuKnockOn",0),
	("DoTauKnockOn",0),
	("DoMu2ePairCreation",1),
	("DoTau2ePairCreation",1),
	("DoMu2muPairCreation",0),
	("DoTau2muPairCreation",0),
	("DoMu2tauPairCreation",0),
	("DoTau2tauPairCreation",0),
	("DoMuPhotoNuclear",1),
	("DoTauPhotoNuclear",1),
	("DoGlashowResonance",1),
	("DoMuDecay",1),
	("DoTauDecay",1),
	("StartLocationID",2)  # 1: from the Earth surface
	# 2: 880m away from the IceCube center
	)

# cascade monte carlo module
tray.AddModule("I3CascadeMCModule", 'cmc') (
	("InputMCTree", "I3MCTree"),
	("OutputMCTree", "CMCTree"),
	('SplitWidth', 2)  # distance between sub-cascades is 2 * X0 ~ 6 * 36cm
	)

tray.AddModule("I3HitConstructorModuleEHE","hit-constructor-ehe")(
	("MaxMCHits", 500000),
	("MCTreeName", "CMCTree")
	)

tray.AddModule("I3RomeoInterfaceModule","romeo")(
	("SkipPhotoCathodeSimulator", False),
	("StoreRomeoInfo", True),                      # Store the summary of Romeo
        ("RomeoOutputTimeResolution", 10.*I3Units.ns), # must same as MCHitBinSize of I3MCHitCompressor
        ("RomeoVerboseLevel", 0)                       # verbose level, 0=silence to 5=trace
	)

#tray.AddModule("I3PMTSimulator","pmt")

tray.AddModule("I3DOMsimulator","domsimulator")(
	("SimulatePedestalDroop",True)
#	("SimulatePedestalDroop",False)
	)

tray.AddModule("I3DOMLaunchCleaning","launchcleaning")(
	("InIceInput","InIceRawData"),
	("IceTopInput","IceTopRawData"),
	("InIceOutput","CleanInIceRawData"),
	("IceTopOutput","CleanIceTopRawData"),
	("FirstLaunchCleaning",True),
	("CleanedKeys",[OMKey(29,59),#Dead/problem list
			OMKey(29,60),
			OMKey(30,23),
			OMKey(30,60),
			OMKey(38,59),
			OMKey(39,22),
			OMKey(50,36),
			OMKey(74,9),
			OMKey(40,51),
			OMKey(40,52),
			OMKey(46,49),
			OMKey(46,50),
			OMKey(46,53),
			OMKey(46,54),
			OMKey(46,55),
			OMKey(46,56),
			OMKey(47,55),
			OMKey(47,56),
			OMKey(56,57),
			OMKey(56,58),
			OMKey(72,37),
			OMKey(72,38),
			OMKey(39,8), #Dark Noise mode only DOMS
			OMKey(50,58),
			OMKey(72,33),
			OMKey(72,34),
			OMKey(72,35),
			OMKey(72,36),
			OMKey(72,43),])
	)

tray.AddModule("I3DOMcalibrator","domcalibrator")(
        ("InputRawDataName","CleanInIceRawData"),
	("CalibrationMode",1),
	("CorrectPedestalDroopDualTau", False)
	)

tray.AddModule("I3RawDataEraser","eraser")(
        ("DataReadoutName","CleanInIceRawData"),
	("eraseFADC", False) # false: keep calibratedFADCs, but delete rawFADCs
	)


tray.AddModule("I3Portia","pulse")(
    ("DataReadoutName", "CleanInIceRawData"),
    ("ATWDPulseSeriesName",         "ATWDPulseSeries"),
    ("ATWDPortiaPulseName",         "ATWDPortiaPulse"),
    ("ATWDWaveformName",            "CalibratedATWD"),
    ("ATWDBaseLineOption",          "eheoptimized"),
    ("FADCBaseLineOption",          "eheoptimized"),
    ("ATWDThresholdCharge",         0.0*I3Units.C),
    ("ATWDLEThresholdAmplitude",    0.5*I3Units.mV),
    ("MakeIceTopPulse", False),
    ("UseFADC", True),
    ("FADCPulseSeriesName",         "FADCPulseSeries"),
    ("FADCPortiaPulseName",         "FADCPortiaPulse"),
    ("FADCWaveformName",            "CalibratedFADC"),
    ("FADCThresholdCharge",         0.0*I3Units.C),
    ("FADCLEThresholdAmplitude",    0.5*I3Units.mV),
    ("MakeBestPulseSeries",         False),
    ("BestPortiaPulseName",         "BestPortiaPulse"),
    #("PMTGain",                     1.0e7)
    )

tray.AddModule("I3EHEFirstGuess", "reco")(
    ("MinimumNumberPulseDom",  2),
    ("OutputFirstguessName",   "OpheliaFirstGuess"),
    ("InputPulseName1",        "ATWDPortiaPulse"),
    ("InputPulseName2",        "FADCPortiaPulse"),
    ("InputLaunchName",        "CleanInIceRawData"),
    ("ChargeOption",           0),
    ("LCOption",               True)
    )

tray.AddModule("I3WeigherModuleJuliet","weight")(
    ("outTreeName",            "JulietTree"),
    ("outFileName",             outrootfile),
    ("inMCHitSeriesName",      "MCHitSeriesMap"),
    ("inDOMLaunchName",        "CleanInIceRawData"),
    ("inMCTreeName",           "I3MCTree"),
    #("inMCTreeName",           "CMCTree"),

    ("doPulseTree",            False),
    ("inAtwdPortiaName",       "ATWDPortiaPulse"),
    ("inFadcPortiaName",       "FADCPortiaPulse"),

    ("doMCHitTree",            False),
    ("McHitCompressionOption", True),

    ("doFirstguessTree",       True),
    ("inFirstguessName",       "OpheliaFirstGuess"),

#  ("doFillWeight",      False),
  ("doFillWeight",        True),
  ("frameMCWeightName",   "JulietWeightDict"),

  ("doReWeigh",           False),
  ("PrimaryParticleType", 0), # 0: neutrino  1: atmosphericMuon
  ("NeutrinoModelList",   [4,7,8,11]),
  ("NDOMthres",           1)

 )

if number == 0:
	tray.AddModule("I3Writer","gcdwriter")(
		("filename", outgcdfile),
		("skipkeys", ["FrameIndex"]),
		("streams",["Geometry", "Calibration", "DetectorStatus"])
		)

tray.AddModule("I3Writer","writer")(
        ("filename", physfile),
	("skipkeys", ["InIceRawData","IceTopRawData"]), # Skip writing the DOMLaunch which
	# is the DOM data before the LC cleaning
	("streams",["Physics"])
	)
tray.AddModule("Dump","dump")
tray.AddModule("TrashCan", "the can")

tray.Execute(eve_num)
tray.Finish()
